﻿// File processor
//    for moraMP4fix_for_NW-A860
//
// Copyright(C) 2014 Sota. All rights reserved.

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.IO;

namespace moraMP4fix_for_NW_A860
{
    class FileProcessing
    {
        /// <summary>
        /// Process one MP4 file
        /// </summary>
        /// <param name="fileName">File name</param>
        /// <param name="gui">GUI</param>
        /// <param name="modified">(OUT) Modified flag</param>
        /// <retunrs>true=error</retunrs>
        public bool ProcessOneFile(string fileName, Form1 gui, out bool modified)
        {
            bool error = false;
            modified = false;

            gui.ShowMessage(fileName);

            byte[] fileData = null;
            int fileSize = 0;
            try
            {
                fileData = File.ReadAllBytes(fileName);
                fileSize = fileData.Count();
            }
            catch (Exception e)
            {
                gui.ShowErrorMessage(global::moraMP4fix_for_NW_A860.Properties.Resources.FileReadError, e);
                error = true;
            }

            if (!error)
            {
                Mp4Processing mp4 = new Mp4Processing();
                if (mp4.CheckMp4File(fileData, fileSize))
                {
                    ClipManager clip = new ClipManager();
                    mp4.ModifyMp4Data(fileData, fileSize, clip, gui);
                    if (clip.GetClipDataCount() > 0)
                    {
                        if (gui.GetBackupFileChecked())
                        {
                            error = MakeBackupFile(fileName, gui);
                        }

                        if (!error)
                        {
                            error = WriteMp4File(fileName, fileData, fileSize, clip, gui);

                            if (!error)
                            {
                                gui.ShowMessage(global::moraMP4fix_for_NW_A860.Properties.Resources.Modified);
                                modified = true;
                            }
                        }
                    }
                    else
                    {
                        gui.ShowMessage(global::moraMP4fix_for_NW_A860.Properties.Resources.NotModified);
                    }
                }
                else
                {
                    gui.ShowMessage(global::moraMP4fix_for_NW_A860.Properties.Resources.NotMp4File);
                }

            }
            return error;
        }

        /// <summary>
        /// Make backup file
        /// </summary>
        /// <param name="fileName">File name</param>
        /// <param name="gui">GUI</param>
        /// <retunrs>true=error</retunrs>
        private bool MakeBackupFile(string fileName, Form1 gui)
        {
            bool error = false;

            string backupFileName = fileName + ".bak";
            if (File.Exists(backupFileName))
            {
                for (int i = 1; ; i++)
                {
                    backupFileName = fileName + ".bak(" + i + ")";
                    if (!File.Exists(backupFileName))
                    {
                        break;
                    }
                }
            }
            try
            {
                File.Copy(fileName, backupFileName);
            }
            catch (Exception e)
            {
                gui.ShowErrorMessage(global::moraMP4fix_for_NW_A860.Properties.Resources.BackupFileMakeError, e);
                error = true;
            }

            return error;
        }

        /// <summary>
        /// Write modified MP4 data to file
        /// </summary>
        /// <param name="fileName">File name</param>
        /// <param name="mp4Data">MP4 data top</param>
        /// <param name="size">MP4 data size</param>
        /// <param name="clip">Clip manager</param>
        /// <param name="gui">GUI</param>
        /// <retunrs>true=error</retunrs>
        private bool WriteMp4File(string fileName, byte[] mp4Data, int size, ClipManager clip, Form1 gui)
        {
            bool error = false;
#if false
            for (int i = 0; i < clip.count(); i++)
            {
                ClipManager.ClipData clipData = clip.get(i);
                gui.showMessage(string.Format("offset=0x{0:x}, size=0x{1:x}", clipData.offset, clipData.size));
            }
#endif

#if true
            int dataOffset = 0;
            int clipDataIndex = 0;

            try
            {
                FileStream fs = new FileStream(fileName, FileMode.Create, FileAccess.Write);
                while (size > 0)
                {
                    if (clipDataIndex < clip.GetClipDataCount())
                    {
                        int writeSize = clip.GetClipData(clipDataIndex).offset - dataOffset;
                        if (writeSize > 0)
                        {
                            fs.Write(mp4Data, dataOffset, writeSize);
                        }
                        dataOffset += writeSize + clip.GetClipData(clipDataIndex).size;
                        size -= writeSize + clip.GetClipData(clipDataIndex).size;
                        clipDataIndex++;
                    }
                    else
                    {
                        fs.Write(mp4Data, dataOffset, size);
                        size -= size;
                    }
                }

                fs.Close();
            }
            catch (Exception e)
            {
                gui.ShowErrorMessage(global::moraMP4fix_for_NW_A860.Properties.Resources.FileWriteError, e);
                error = true;
            }

            return error;
#endif
        }
    
    }
}
